# Connect2id Server Change Log

## 6.6.2 (2017-04-26)

### General

* Switches to constant time comparison in the master web API token validation 
  routines to guard against side-channel / timing attacks. The token values are 
  salted and hashed with SHA-256 before comparison for additional protection.
  
* Switches to constant time comparison in the client_secret validation routine
  to guard against side-channel / timing attacks. The client_secret values are 
  hashed with SHA-256 before comparison for additional protection.


### Configuration

* /WEB-INF/jwkSet.json

    * Checks the supplied elliptic curve (EC) keys on Connect2id server startup 
      to ensure the public 'x' and 'y' parameters match the curve (P-256, P-384 
      or P-521).


### Web API

* No changes


### Bug fixes

* Fixes intermittent duplication of client objects when listing all clients via 
  the client registration web API (HTTP GET) for Infinispan setups in 
  invalidation mode with Redis as primary cache and an SQL database as 
  persistence store (issue #273).


### Dependencies

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.26

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.37

* Upgrades to com.nimbusds:oauth2-authz-store:5.14.2

* Upgrades to com.nimbusds:common:2.5




## 6.6.1 (2017-04-12)

### Configuration

* No changes


### Web API

* No changes


### Bug fixes

* For client_secret provisioning for client_secret_jwt authentication with 
  HS384 and HS512 at the token endpoint to ensure the client secret is of 
  sufficient length for the HMAC algorithm (issue #272).


### Dependencies

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.24.2

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.35

* Upgrades to mysql:mysql-connector-java:5.1.41

* Upgrades to Log4j 2.8.2



## 6.6 (2017-03-20)

### General

* Adds comprehensive support for importing subject sessions and persisted 
  authorisation records from another Connect2id server.

### Configuration

* /WEB-INF/sessionStore.properties

    * Adds new optional sessionStore.acceptLegacySIDs configuration property. 
      Set to "true" to enable import of sessions from Connect2id server 
      versions 5x and older. Defaults to "false".


### Web API

* /session-store/rest/v2 POST

    * Adds new optional "Legacy-SID" header parameter to enable import of a 
      subject session from a Connect2id server that doesn't support 
      HMAC-protected SIDs (version 5x and older). Note that the Connect2id 
      server must also be configured to accept legacy SIDs with 
      sessionStore.acceptLegacySIDs = true.

* /authz-store/rest/v2/authorizations POST

    * Adds new optional "import" query parameter to enable import of a 
      long-lived authorisation from another Connect2id server instance.


### Bug fixes

* None


### Dependencies

* Upgrades to com.nimbusds:oauth2-authz-store5.14.1

* Upgrades to com.nimbusds:oidc-session-store:5.2.1




## 6.5 (2017-03-15)

### Configuration

* No changes


### Web API

* /token/introspect

    * Adds an optional non-standard (see RFC 7662) "revoke" query parameter 
      (defaults to "false") to facilitate single use of identifier-based 
      access tokens. Causes the access token to be automatically deleted from 
      the store after successful inspection. Has no effect with self-contained 
      (JWT-encoded) access tokens. Identical with the optional "revoke" query 
      parameter of /authz-store/rest/v2/inspection (introduced in Connect2id 
      server 6.4).

* /session-store/rest/v2

    * Permits addition of subject sessions created in the past, in order to 
      facilitate the correct import of sessions from another Connect2id server. 


### Bug fixes

* Fixes a bug that caused the set authentication lifetime of a subject session 
  to incorrectly cause its expiration (issue session-store/50).


### Dependencies

* Upgrades to com.nimbusds:oidc-session-store:5.1




## 6.4 (2017-03-08)

### General

* Adds new optional configuration file for defining custom scope value to 
  OpenID Connect claim mappings.

* Adds support for implicit consent of OpenID Connect claims.

* Adds a new optional "revoke" parameter to the token introspection call to 
  facilitate single use of identifier-based access tokens where required by the 
  resource server / application.
  
* Adds endpoint for scraping the Connect2id server metrics in Prometheus 
  (https://prometheus.io) format.

* Expiration of subject (end-user) sessions is now handled by Infinispan, 
  simplifying configuration (see below) and improving the performance of 
  session retrieval.

* Updates the Infinispan configuration files.


### Configuration

* /WEB-INF/customClaimsMap.properties

    * New optional configuration file for defining custom scope value to OpenID 
      Connect claim mappings. Can be used to automatically expand selected 
      custom scope values to one or more custom claim names, e.g. 
      "my_scope_value" to claim names "claim_a", "claim_b", etc.

* /WEB-INF/oidcProvider.properties

    * Adds new optional "op.authz.alwaysPromptForAuth" configuration property, 
      defaults to "false". If "true" the Connect2id server will always prompt 
      for authentication, even if the end-user is currently authenticated (by 
      means of a valid session cookie), or "prompt=none" was requested by the 
      client. Intended to facilitate authentication step-up by using selected 
      scope values.

* /WEB-INF/sessionStore.properties

    * The sessionStore.purgeInterval configuration property is removed. 
      Expiration of subject (end-user) sessions is now handled by Infinispan. 
      The expiration interval can be fine tuned by changing the expiration 
      interval attribute of "sessionStore.sessionMap" of the chosen Infinispan 
      XML configuration (infinispan-<backend>.xml).

* /WEB-INF/jose.properties

    * Adds support for overriding the PKCS#11 configuration via Java system 
      properties.
      
* /WEB-INF/infinispan-mysql-redis.xml

    * Adds new configuration for using Infinispan in invalidation mode, with 
      MySQL as the persistence store and Redis as the primary in-memory / cache 
      store.

* /WEB-INF/infinispan-postgres95-redis.xml

    * Adds new configuration for using Infinispan in invalidation mode, with 
      PostgreSQL 9.5+ as the persistence store and Redis as the primary 
      in-memory / cache store.

* /WEB-INF/infinispan-ldap-redis.xml

    * Renames the previous configuration file for using Infinispan in 
      invalidation mode with LDAP as the persistence store and Redis as the 
      primary in-memory / cache store.
      
* /WEB-INF/infinispan-*.xml

    * Declares the required application specific AdvancedExternalizer instances 
      in the XML configuration (see issue server/253 below).

* /WEB-INF/web.xml

    * Configures eager servlet loading. 


### Web API

* /authz-sessions/rest/v3, /authz-sessions/rest/v2

    * Adds support for implicit consent of OpenID Connect claims.

* /authz-store/rest/v2/inspection

    * Adds an optional "revoke" query parameter (defaults to "false") to 
      facilitate single use of identifier-based access tokens. Causes the 
      access token to be automatically deleted from the store after successful 
      inspection. Has no effect with self-contained (JWT-encoded) access 
      tokens.

* /monitor/v1/metrics/prometheus

    * Adds new resource for retrieving Connect2id server metrics in Prometheus 
      (https://prometheus.io) format (exported from Dropwizard). Requires a  
      bearer access token, which is configured in /WEB-INF/monitor.properties. 


### Bug fixes

* Fixes processing of refresh token grant requests from public OAuth 2.0 
  clients to ensure a mismatch of the top-level client_id and the client_id 
  encoded in the refresh token produces an invalid_grant error (issue 
  server/254).
  
* Switches to XML-based declaration of the application specific Infinispan 
  AdvancedExternalizer classes to address startup situations when the 
  programmatically configured externalisers don't get picked up by Infinispan 
  (issue server/253).
  
* Fixes handling of illegal URL-encoding in client secret basic authentication 
  HTTP Authorization headers so that instead of failing with HTTP 500 status 
  code a proper HTTP 400 status and message is returned (issue oidc-sdk/208).
  
* Prevents inconsistent direct authorisation requests for a refresh token with 
  disabled authorisation persistence (long_lived = false). Such requests will 
  now produce a HTTP 400 status code with the error message "Invalid request: 
  Refresh token issue requires the authorization to be persisted with 
  long_lived=true" (issue server/262).
  
* Fixes a bug that prevented the timely removal of authorisation sessions in 
  Infinispan invalidation mode (issue server/265).

* Removes stray System.out.println in userInfo endpoint code (issue 
  server/261).


### Dependencies

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.24

* Upgrades to com.nimbusds:oauth2-authz-store:5.13.1

* Upgrades to com.nimbusds:oidc-session-store:5.0

* Upgrades to com.nimbusds:nimbus-jwkset-loader:1.3

* Upgrades to com.nimbusds:common:2.4

* Upgrades to com.unboundid:unboundid-ldapsdk:3.2.1




## 6.3 (2017-01-30)

General

* Adds comprehensive support for signing issued ID and self-contained access 
  tokens with RSA and EC keys stored in a PKCS#11 compliant Hardware Security 
  Module (HSM). Supports automatic key rollover based on the not-before and 
  not-after dates specified in the X.509 certificate of each PKCS#11 based 
  RSA and EC key intended for signing.


### Configuration

* /WEB-INF/jose.properties
    * Introduces new optional configuration file for loading RSA and EC signing 
      JSON Web Keys (JWK) from a PKCS#11 compliant HSM.

* /WEB-INF/hsm.cfg
    * Adds sample configuration file for the Java SUN PKCS#11 security provider
      (to enable loading of a PKCS#11 compliant HSM).

* /WEB-INF/web.xml
    * Adds com.nimbusds.jose.jwk.loader.JWKSetLoader listener.


### Web API

* No changes


### Logging

* Logs a detailed message at level "INFO" for a token request with JWT client 
  authentication (client_secret_jwt, private_key_jwt) where the JWT has expired 
  or the claims validation failed (issue sdk/204).


### Bug fixes

* Outputs a proper OAuth 2.0 invalid_client error if a token request includes 
  multiple client authentication methods (issue sdk/203).

* Fixes a bug which caused the BouncyCastle JCA provider to be loaded if "none"  
  is detected among the supported ID token JWS algorithms (non-critical).


### Dependencies

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.21

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.34.1

* Upgrades to com.nimbusds:oauth2-authz-store:5.11.1

* Upgrades to com.nimbusds:oidc-session-store:4.14.3

* Adds new com.nimbusds:nimbus-jwkset-loader:1.2.2 dependency.

* Upgrades to org.bouncycastle:bcprov-jdk15on:1.56

* Upgrades to org.bouncycastle:bcpkix-jdk15on:1.56

* Upgrades to Infinispan 8.2.6.Final

* Upgrades to JAX-RS Jersey 2.25.1

* Upgrades to Log4j 2.8




## 6.2 (2017-01-12)

### Configuration

* No changes


### Web API

* /authz-sessions/rest/v3/
    * Enables storage of additional data in the authorisation session, to 
      enable use cases such as a stateless login front-end that needs to 
      perform a redirection to an external service or IdP as part of the 
      authentication or consent process.
        * Adds new optional "data" parameter of type JSON object to the 
          authorisation session.
        * The optional "data" can be set with the initial POST request for a 
          new authorisation session, or with a dedicated PUT request to the 
          authorisation session data resource.
        * The optional "data" can be retrieved with a GET for the authorisation 
          session, or directly from the authorisation session data resource.

* /monitor/v1/metrics
    * Adds Redis store connection pool metrics (of type gauge):
        * "[infinispan-cache-name].redisStore.numActiveConnections" -- the 
          number of active Redis client connections in the pool.
        * "[infinispan-cache-name].redisStore.numIdleConnections" -- the number 
          of idle Redis client connections in the pool.
        * "[infinispan-cache-name].redisStore.numWaitingForConnection" -- the 
          number of threads waiting for a Redis client connection.
        * "[infinispan-cache-name].redisStore.meanWaitingTimeForConnectionMs"
          -- the mean time waiting to borrow a Redis client connection from the 
          pool, in milliseconds.
        * "[infinispan-cache-name].redisStore.maxWaitingTimeForConnectionMs"
          -- the maximum time waiting to borrow a Redis client connection from 
          the pool, in milliseconds.

### Bug fixes

* None


### Dependencies

* Upgrades to Nimbus JOSE+JWT 4.34.

* Upgrades to Redis Store 8.2.1 (private Connect2id release)




## 6.1.3 (2016-12-31)

### Configuration

* No changes


### Web API

* /monitor/v1/metrics
    * Adds Redis store connection pool metrics (of type gauge):
        * "[infinispan-cache-name].redisStore.numActiveConnections" -- the 
          number of active Redis client connections in the pool.
        * "[infinispan-cache-name].redisStore.numIdleConnections" -- the number 
          of idle Redis client connections in the pool.
        * "[infinispan-cache-name].redisStore.numWaitingForConnection" -- the 
          number of threads waiting for a Redis client connection.
        * "[infinispan-cache-name].redisStore.meanWaitingTimeForConnectionMs"
          -- the mean time waiting to borrow a Redis client connection from the 
          pool, in milliseconds.
        * "[infinispan-cache-name].redisStore.maxWaitingTimeForConnectionMs"
          -- the maximum time waiting to borrow a Redis client connection from 
          the pool, in milliseconds.

### Bug fixes

* None


### Dependencies

* Upgrades to OAuth 2.0 / OpenID Connect SDK 5.20

* Upgrades to Redis Store 8.2.1 (private Connect2id release)



## 6.1.2 (2016-12-15)

### General 

* For Connect2id servers using MySQL as backend, updates the create table 
  statements to explicitly make UTF-8 the default character set. The VARCHAR(x) 
  sizes of key fields are adjusted where needed to accommodate the MySQL row 
  restriction of 65535 bytes. Changes the type of "clients" fields 
  "client_name", "client_uri", "logo_uri", "policy_uri", "tos_uri" and "data"
  from VARCHAR(X) to JSON. Changes the type of "id_access_tokens" fields "uip"
  and "dat" to JSON. 
  
* Improves error reporting on malformed client secret basic authentication at 
  the token endpoint, includes reference to RFC 6749, section 2.3.1 (issue 
  oidc-sdk/201).


### Configuration

* /WEB-INF/infinispan-mysql.xml

    * Updates the MySQL JDBC URL to set the connection encoding to UTF-8, e.g.
      "jdbc:mysql://localhost/c2id?useUnicode=yes&characterEncoding=UTF-8"
    
    
### Web API
    
* No changes


### Bug fixes

* None


### Dependencies

* Upgrades to com.nimbusds:oauth2-authz-store:5.10

* Upgrades to com.nimbusds:oidc-session-store:4.13

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.19.2

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.33

* Upgrades to com.nimbusds:common:2.2

* Upgrades to com.nimbusds:infinispan-cachestore-ldap:2.2.2

* Upgrades to com.nimbusds:infinispan-cachestore-sql:2.5.7



## 6.1.1 (2016-11-29)

### Configuration

* /WEB-INF/infinispan-h2.xml

    * Updates the H2 dataSource.url properties to permit Apache Tomcat startup 
      from any current directory so that the H2 database files can be 
      correctly located and loaded.
    
    
### Web API
    
* No changes


### Bug fixes

* None


### Dependencies

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.18.1

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.29

* Upgrades to org.infinispan:infinispan-core:8.2.5

* Upgrades to com.h2database:h2:1.4.193

* Upgrades org.postgresql:postgresql:9.4.1212

* Upgrades to mysql:mysql-connector-java:5.1.40

* Upgrades JAX-RS Jersey to 2.24.1

* Upgrades to Apache Commons IO 2.5

* Upgrades Log4j to 2.7



## 6.1 (2016-11-16)

### Configuration

* /WEB-INF/infinispan-redis.xml

    * Provides separate Java system properties for the Redis instance intended 
      for storage in-memory data (redisMapHost, redisMapPort) and the Redis 
      instance intended for caching of data (redisCacheHost, redisCachePort).
      See http://connect2id.com/products/server/docs/guides/two-tier-caching
    
    
### Web API
    
* /clients/

    * Returns detailed error_description on a client registration request for 
      pairwise subject identifiers with one or more redirect_uris missing from 
      the sector_identifier_uri document. 

* /monitor/v1/metrics

    * Adds new clientStore.numRegistrations gauge to track the number of OAuth 
      2.0 / OpenID Connect client registrations.
      
    * Deprecates the clientStore.numCachedRegistrations gauge, use 
      clientStore.numRegistrations instead.

    * Improves responsiveness of the following object count metrics when Redis 
      is utilised as primary or passivating in-memory session store: 
      sessionStore.numSessions, authzStore.numAuthzCodes, 
      authzStore.numIdAccessTokens, authzStore.numLongLivedAuthorizations, 
      authzStore.numRevocationJournalEntries, authzSessionStore.numSessions and 
      clientStore.numRegistrations.


### Bug fixes

* None


### Dependencies

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.17.6

* Upgrades to com.nimbusds:oauth2-authz-store:5.9.3

* Upgrades to com.nimbusds:oidc-session-store:4.12.2

* Upgrades to com.nimbusds:infinispan-cachestore-common:1.3

* Upgrades to com.nimbusds:infinispan-cachestore-ldap:2.2.1

* Upgrades to com.nimbusds:infinispan-cachestore-sql:2.5.6

* Upgrades to com.nimbusds:infinispan-cachestore-redis:8.2

* Upgrades to redis.clients:jedis:2.8.2



## 6.0 (2016-10-29)

### General

* Switches to Infinispan's persistent store APIs to enable support of other
  Connect2id server backends besides LDAP, such as PosgreSQL, MySQL, H2, flat
  file storage, Redis, AWS Elasticache and AWS S3.

* Adds support for ID tokens signed with EC DSA (ES256, ES384 and ES512).

* Adds support for UserInfo JWTs signed with EC DSA (ES256, ES384 and ES512).
  
* Adds support for encrypted ID tokens. A client can receive ID tokens 
  encrypted with a public RSA or EC key it has registered with the Connect2id 
  server (via the jwks or jwks_uri client registration parameter), or with a 
  symmetric AES key derived from its client_secret.
  
* Adds support for encrypted UserInfo. A client can receive UserInfo JWTs 
  encrypted with a public RSA or EC key it has registered with the Connect2id 
  server (via the jwks or jwks_uri client registration parameter), or with a 
  symmetric AES key derived from its client_secret.

* Adds support for registering OpenID Connect / OAuth 2.0 clients with a JWK
  set specified by URI (jwks_uri). Intended for handling private_key_jwt client
  authentication, self-issued JWT and SAML 2.0 bearer grants, signed request 
  objects, encrypted ID tokens and encrypted UserInfo responses. The downloaded
  client JWK set is cached by the Connect2id server to ensure quick processing
  of requests that rely on remote JWKs.

* Updates handling of the optional max_age OpenID authentication request
  parameter. If max_age is set to zero (max_age=0) the Connect2id server will
  prompt the end-user for (re)authentication, even if the end-user has an
  existing valid session with the Connect2id server. This is equivalent to
  setting the optional prompt OpenID authentication request parameter to login
  (prompt=login). See issue #993 with the OpenID Connect working group
  (https://bitbucket.org/openid/connect/issues/993).

* Adds support for pairwise subject (end-user) identifiers, based on 
  deterministic AES encryption in SIV mode (see RFC 5297).

* Adds an extra layer of security to issued session IDs, authorisation codes
  and identifier-based access tokens by provisioning them with a SHA-256 based
  Message Authentication Code (MAC) (truncated to 128 bits). The HMAC enables 
  detection of online guessing attacks and may also be used to block certain 
  DoS attacks the utilise the above objects.

* Switches to new JSON-based marshalling of Connect2id server objects stored in
  Infinispan.
  
* Provides claims sources and custom OAuth 2.0 grant handlers with access to 
  the Infinispan cache manager to enable custom cache / map creation (issue 
  server/234).

* Refines diagnostic messages on a client registration error due to invalid or 
  inconsistent client metadata.

* Refines logging of invalid client authentication attempts at the token
  endpoint (issue server/199).
  

### Configuration

* /WEB-INF/jwkSet.json

    * All JSON Web Keys (JWKs) in the Connect2id server JWK set must have a key
      identifier ("kid" JWK parameter) and specify the intended key use ("use"
      JWK parameter).

    * All RSA JWKs intended for signing of objects (ID tokens, UserInfo JWTs 
      and self-contained access tokens) must have their use parameter set to
      "sig" (signature). Required to prevent mix up with future RSA JWKs
      intended for encrypting objects to the Connect2id server.
      
    * Adds support for optional EC JWKs with curves P-256, P-384 and P-521 for 
      signing of ID tokens and UserInfo JWTs. The EC JWKs must have a key 
      identifier ("kid" JWK parameter) and be flagged for specify signature use 
      ("use":"sig").

    * All AES JWKs intended for symmetric encryption of self-contained access 
      tokens must have their use parameter set to "enc" (encryption). Required 
      to prevent mix up with the introduced secret JWKs for HMAC protection and 
      pairwise subject encryption (see below).

    * Adds new secret JSON Web Key (JWK) with ID "hmac" and use "sig" to
      facilitate HMAC-SHA256 protection of session IDs, authorisation codes and
      identifier-based access tokens. The HMAC key size must be 256 bits.

    * Adds new symmetric JSON Web Key (JWK) with ID "subject-encrypt" to 
      facilitate AES SIV-mode (RFC 5297) encryption of pairwise subject 
      identifiers. The accepted AES key sizes are 128, 192 and 256 bits.

* /WEB-INF/oidcProvider.properties

    * Extends the op.idToken.jwsAlgs configuration setting to include support 
      for ID tokens signed with EC DSA (ES256, ES384 and ES512).
      
    * Adds new op.idToken.jweAlgs configuration setting for the enabled JWE key 
      management algorithms for applying optional additional encryption to 
      issued ID tokens. All standard symmetric and public key JWE algorithms 
      are supported: RSA1_5, RSA-OAEP, RSA-OAEP-256, ECDH-ES, ECDH-ES+A128KW, 
      ECDH-ES+A192KW, ECDH-ES+A256KW, dir, A128KW, A192KW, A256KW, A128GCMKW, 
      A192GCMKW and A256GCMKW.
      
    * Adds new op.idToken.jweEncs configuration setting for the enabled JWE 
      content encryption methods for applying optional additional encryption to 
      issued ID tokens. All standard JWE methods are supported: A128CBC-HS256, 
      A192CBC-HS384, A256CBC-HS512, A128GCM, A192GCM and A256GCM.
      
    * Adds new op.authz.requestJWSAlgs configuration setting for the accepted  
      JWS algorithms for signed OpenID authentication requests passed with the 
      optional request_uri or request parameter. The following JWS algorithms 
      are supported: HS256, HS384, HS512, RS256, RS384, RS512, PS256, PS384, 
      PS512, ES256, ES384, ES512 and none.

    * Adds new op.reg.resourceRetriever.httpConnectTimeout configuration
      setting for the HTTP connect timeout (in milliseconds) when retrieving
      client resources (JWK sets and request objects specified by URI). Zero
      implies no timeout. Defaults to 250 ms.

    * Adds new op.reg.resourceRetriever.httpReadTimeout configuration setting
      for the HTTP read timeout (in milliseconds) when retrieving client
      resources (JWK sets and request objects specified by URI). Zero implies
      no timeout. Defaults to 250 ms.
 
    * Removes the op.reg.preloadCache, op.reg.ldapServer.*, op.reg.ldapUser.*,
      op.reg.ldapDirectory.*, op.reg.customTrustStore.* and 
      op.reg.customKeyStore.* settings for configuring an LDAP store for the 
      OpenID Connect and OAuth client registration. Configuration of the 
      client registration store is now assumed by the Infinispan.
      
    * Extends the op.userinfo.jwsAlgs configuration setting to include support 
      for UserInfo JWTs signed with EC DSA (ES256, ES384 and ES512).
      
    * Adds new op.userinfo.jweAlgs configuration setting for the enabled JWE 
      key management algorithms for applying optional additional encryption to 
      issued UserInfo JWTs. All standard symmetric and public key JWE 
      algorithms are supported: RSA1_5, RSA-OAEP, RSA-OAEP-256, ECDH-ES, 
      ECDH-ES+A128KW, ECDH-ES+A192KW, ECDH-ES+A256KW, dir, A128KW, A192KW, 
      A256KW, A128GCMKW, A192GCMKW and A256GCMKW.
      
    * Adds new op.userinfo.jweEncs configuration setting for the enabled JWE 
      content encryption methods for applying optional additional encryption to 
      issued UserInfo JWTs. All standard JWE methods are supported: 
      A128CBC-HS256, A192CBC-HS384, A256CBC-HS512, A128GCM, A192GCM and 
      A256GCM.
      
* /WEB-INF/authzStore.properties

    * Removes the authzStore.options.preloadCache, authzStore.ldapServer.*, 
      authzStore.ldapUser.*, authzStore.ldapDirectory.*, 
      authzStore.customTrustStore.* and authzStore.customKeyStore.* settings 
      for configuring an LDAP store for the long-lived (persisted) OpenID 
      Connect and OAuth authorisations. Configuration of the long-lived 
      authorisations store is now assumed by Infinispan.
      
    * Adds new authzStore.options.discardedAttributes settings to list the
      optional authorisation attributes that are to be automatically discarded 
      from new authorisation objects, in order to conserve cache memory and 
      backend storage.
     
    * Adds the "iat" (issued-at) claim to the list of recommended JWT claims to
      include in self-contained access tokens. Required to facilitate correct 
      inspection of revoked self-contained access tokens.

* /WEB-INF/infinispan.xml

    * Updates all advanced externalisers for Connect2id server data stored in 
      Infinispan. The externalisers are now declared programmatically, instead 
      of using the Infinispan XML configuration. 

    * Adds new "clients.remoteJWKSetCache" cache.
    
    * Replaces the "op.regCache" cache with "clients.registrationsMap".
    
    * Replaces the "op.regTokenCache" cache with 
      "op.clientRegTokenMap".
    
    * Replaces the "authzStore.authzCache" cache with
      "authzStore.longLivedAuthzMap".



### Web API

* /.well-known/openid-configuration

    * Advertises support for pairwise subject identifiers 
      (subject_types_supported).

* /clients/

    * Supports registration of OAuth 2.0 and OpenID Connect clients with a JWK
      set specified by URL (jwks_uri).

    * Supports registration of OpenID Connect clients for pairwise subject 
      identifiers. OpenID Connect clients can be registered for pairwise 
      subject identifiers by setting the "subject_type" parameter to 
      "pairwise"; if more than one redirection URI is specified in the 
      "redirect_uris" parameter, a "sector_identifier_uri" parameter must also 
      be provided.

* /session-store/rest/v2/sessions

	* Support for the optional "SID" header parameter on POST new session is
	  removed, due to the introduction of HMAC secured session IDs. The key
	  portion of the session ID can be set with the new "SID-Key" header
	  parameter; the Connect2id server will then return the final SID (session
	  key with appended HMAC).

* /authz-store/rest/v2/revocation

    * Extends the effect of revocation to issued access tokens as well.
      Previously, revocation was limited to deleting the long-lived
      authorisations and any associated refresh tokens that match the
      revocation spec (subject, actor, client_id).

* /authz-store/rest/v2/inspection

	* Access tokens that fall under the scope of a previous revocation event
	  will be resolved as invalid. Previously, access tokens will appear as
	  valid until they expire, unless the access token is identifier-based and
	  had been explicitly revoked.

* /monitor/v1/metrics

    * Replaces the "authzStore.numCachedAuthorizations" metric with
      "authzStore.numLongLivedAuthorizations".
      
    * Adds new "authzStore.numRevocationJournalEntries" metric for the number 
      of revocation journal entries.
      
    * Adds new "clientStore.numCachedRemoteJWKSets" metric for the number of
      cached client JWK sets.
      
    * Adds new "clientStore.numCachedRemoteRequestObjects" metric for the 
      number of cached OpenID requests objects.
    
    
### Bug fixes

* Adds check to prevent client registration of redirect_uris with a fragment 
  component (issue sdk/188).
  
* Fixes NPE in the LDAP connector when the asynchronous LDAP result listener 
  receives a notification before the associated timer context is cached (issuer 
  authz-store/117).

* Fixes Log4j class loading conflict caused by shadowing of Log4j by the
  Infinispan Embedded package (iss #219).
  
* Enables stack trace logging on Infinispan cache manager launch failure.

    
### Dependencies

* Upgrades to com.nimbusds:c2id-server-sdk:3.10.1

* Upgrades to com.nimbusds:oauth2-authz-store:5.9.1

* Upgrades to com.nimbusds:oidc-session-store:4.11

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.17.2

* Upgrades to com.nimbusds.nimbus-jose-jwt:4.27

* Upgrades to org.bouncycastle:bcprov-jdk15on:1.55

* Upgrades to com.nimbusds:common:2.1

* Upgrades to Infinispan 8.2.4.Final

* Upgrades to JAX-RS Jersey 2.23.2

* Upgrades to com.unboundid:unboundid-ldapsdk:3.2.0

* Upgrades to Log4j 2.6.2

* Adds JHades dependency to diagnose class loader issuer at runtime (issue
  server/219).




## 5.0 (2016-05-17)

### General

* Upgrades to Java 8, which becomes the required runtime version.

* Upgrades to Infinispan 8.2, for reduced resource consumption, reduced 
  locking, separate thread pool for asynchronous operations, global state 
  persistence, performance improvements, and utilisation of Java 8 APIs. 

* Adds support for the optional form_post response mode, as specified in OAuth 
  2.0 Form Post Response Mode, 1.0. The specification is available at  
  https://openid.net/specs/oauth-v2-form-post-response-mode-1_0.html
  
* Adds support for implementing custom (experimental) OAuth 2.0 response modes, 
  such as based on CORS Ajax or window.postMessage.


### Configuration

* /WEB-INF/oidcProvider.properties 
 
    * Adds new op.authz.responseModes configuration setting for the supported 
      OAuth 2.0 response modes. Besides the standard query, fragment and 
      form_post, it may also include custom (non-standard, experimental) 
      response modes, provided the authorisation session front-end (the login 
      page) is able to support them. Authorisation requests with a response 
      mode that is not listed as supported will be rejected without a 
      redirection back to the OAuth client.

    * Adds new op.reg.ldapServer.connectionPoolInitialSize configuration 
      setting for the initial number of pooled LDAP connections to be created
      when the Connect2id server is started. The accepted range is between zero 
      and the maximum LDAP connection pool size (inclusive). Set it to zero to 
      enable startup of the Connect2id server without the backend LDAP
      directory being available.
      
* /WEB-INF/authzStore.properties 
 
    * Adds new authzStore.ldapServer.connectionPoolInitialSize configuration 
      setting for the initial number of pooled LDAP connections to be created
      when the Connect2id server is started. The accepted range is between zero 
      and the maximum LDAP connection pool size (inclusive). Set it to zero to 
      enable startup of the Connect2id server without the backend LDAP 
      directory being available.
      
* /WEB-INF/monitor.properties 
 
    * Allows Java system properties to override individual properties in the 
      configuration file.
      
* /WEB-INF/infinispan.xml

    * Updates the configuration to match the 8.2 Infinispan XML schema.
      
    * Sets the sessionStore.sessionMap, sessionStore.subjectMap, 
      authzStore.accessTokenMap, authzStore.authzCache, op.regCache and
      op.regTokenCache Infinispan maps / caches to asynchronous mode.
      
    * Asynchronous marshalling is no longer supported in Infinispan 8, and is
      removed from the configuration.
      
* /WEB-INF/ldapClaimsSource.properties

    * Adds new op.ldapClaimsSource.server.connectionPoolInitialSize 
      configuration setting for the initial number of pooled LDAP connections 
      to be created when the Connect2id server is started. The accepted range  
      is between zero and the maximum LDAP connection pool size (inclusive). 
      Set it to zero to enable startup of the Connect2id server without the 
      backend LDAP directory being available.
      

### Web API

* New authorization session API /authz-session/v3

    * A final response (with the encoded OAuth 2.0 response parameters, such
      as authorisation code, state, etc) is conveyed by means of a dedicated 
      JSON object, instead of a HTTP 302 redirection with Location header. This 
      enables handling of the optional OAuth 2.0 form_post response mode, as 
      well as implementation of custom (experimental) response modes, such as 
      based on CORS Ajax or window.postMessage. See the online documentation on 
      v3 of the authorisation session API for more information.
      
    * A non-redirectable error response (e.g. due to a client request with 
      invalid client_id or redirect_uri) is conveyed by means of a HTTP 200 
      with JSON object with "type":"error", following the semantics of 
      "type":"auth", "type":"consent" and "type":"response" messages. See the
      online documentation on v3 of the authorisation session API for more 
      information.
    
    * Custom OpenID authentication request parameters get included in the 
      authorisation session, and can be obtained by making a HTTP GET for its
      JSON object representation. Any custom authentication request parameters
      will be included in the auth_req member of the authorisation session, 
      alongside response_type, client_id, redirect_uri, scope, etc. This 
      permits processing of custom and experimental OpenID authentication 
      parameters that are not otherwise supported.
       
* Authorization session API /authz-session/v2

    * V2 of the authorisation API, introduced in Connect2id server v 3.0, 
      remains available.
    
    
### Bug fixes

* Fixes Infinispan serialisation of an explicitly set response_mode in 
  authorisation sessions (issue server/203).

    
### Dependencies

* Upgrades to com.nimbusds:oauth2-authz-store:4.5

* Upgrades to com.nimbusds:oidc-session-store:4.0.1

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.8.1

* Upgrades to com.nimbusds.nimbus-jose-jwt:4.16.2

* Upgrades to com.nimbusds:common:1.102

* Upgrades to org.infinispan:infinispan-embedded:8.2.1.Final

* Upgrades to com.unboundid:unboundid-ldapsdk:3.1.1



## 4.1 (2016-03-04)

### Configuration

* /WEB-INF/infinispan.xml

      * Replaces the advanced externaliser classes 
        com.nimbusds.sessionstore.impl.ext.SubjectSessionExternalizerV1 and
        com.nimbusds.sessionstore.impl.ext.SubjectSessionExternalizerV2 with
        com.nimbusds.sessionstore.impl.ext.SubjectSessionExternalizer (see 
        issue session-store/27).


### Web API

* Session Store API /session-store/rest/v2:
 
      * The POST method adds support for an optional "SID" header to allow 
        creation of new subject (end-user) sessions with a predefined session 
        identifier (SID). Intended to assist migration of sessions (issue 
        session-store/28).
        
* Session Store API /session-store/rest/v2/sessions/count:
 
      * Modifies the behaviour of the GET method to only return the number of
        session present in memory, in order to avoid hitting the cache loader
        (in cases where the session store is configured to overflow or backup 
        sessions to disk) (issue session-store/29).
        

### Bug fixes

* Fixes a serialisation bug that affected Infinispan replication of subject 
  (end-user) sessions between Connect2id server nodes due to non-deterministic
  selection of versioned serialisers for session objects by Infinispan (issue 
  session-store/27). The bug fix necessitated changes to the Infinispan 
  configuration (see above). Live roll-over from previous Connect2id server 
  versions to 4.1 will not migrate existing end-user sessions automatically; 
  these may however be copied by an external job via the updated session store 
  web API.
  
  

### Dependencies

* Upgrades to com.nimbusds:oidc-session-store:3.4.2

* Upgrades to com.nimbusds:oauth2-authz-store:4.1

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.5.1

* Upgrades to org.glassfish.jersey.containers:jersey-container-servlet:2.22.1




## 4.0.1 (2016-02-12)

### Configuration

* /WEB-INF/oidcClientInfoMap.json

      * Corrects the entity mapping between the optional JSON object member 
        "jwks" and the LDAP attribute "oauthJWKSet". See issue server/194 for
        more information.


### Web API

* No changes
        

### Bug fixes

* Fixes a bug that affected recreation of registered client JWK sets from 
  persisted LDAP entries (issue server/194).
  
* Prevent registration of custom client "data" parameters that are not a JSON
  object entity (issue server/195).


### Dependencies

* No changes




## 4.0 (2016-02-11)

### Configuration

* /WEB-INF/authzStore.properties

      * authzStore.accessToken.selfContainedClaims -- Adds support for an
        optional "act" (actor) JWT claim in issued self-contained access 
        tokens. This claim can be used to indicate the authorised actor in 
        impersonation (act-as) and delegation (on-behalf-of) cases. Its 
        semantics is defined in draft-ietf-oauth-token-exchange-03, section 
        4.1.
      
      * authzStore.ldapDirectory.attributes.act -- New required configuration 
        property to specify an LDAP attribute for representing impersonated and
        delegated authorisations. The LDAP attribute holds the identity of the 
        impersonating or delegating subject (actor) and is intended to be used 
        as part of the LDAP entry's RDN. Set to "authzActor".
      
* /WEB-INF/oidcClientInfoMap.json

      * Adds a new entity mapping between the optional JSON object member 
        "data" and the LDAP attribute "oauthClientData". See the client
        registration web API change log for more information.


### Web API

* /clients

      * Adds support for registering a custom "data" client metadata parameter,
        represented by a JSON object entity. Registration of this parameter 
        requires the master registration token or an initial registration token 
        with a "client-reg:data" scope.
        
* /authz-sessions/rest/v2

      * Adds an optional "impersonated_sub" string parameter to the consent 
        object representation. If set specifies a subject (end-user) to
        impersonate or delegate by means of the issued ID, access and refresh
        tokens. May be used to enable privileged users to login into clients 
        and access protected resources under a different identity. The actual 
        authorised subject (the impersonator) will be indicated in the custom 
        "act" -> "sub" ID and access token (JWT) claim.
         
      * The optional "id_token" -> "impersonated_sub" parameter introduced in
        Connect2id server 3.x to enable limited ID token impersonation is 
        deprecated in favour of the new top-level "impersonated_sub" parameter
        (see above).

      * Adds optional "impersonated_sub" string parameter to the authentication 
        and consent prompt representations. If set indicates that the prompt is
        for a previously impersonated subject (end-user), as implied by an ID 
        token hint submitted by the client. See the OpenID authentication 
        request documentation for the "id_token_hint" parameter for more 
        information.
        
* /authz-store/rest/v2

      * Adds an "act" (actor) parameter to the authorisation representation to
        support impersonation (act-as) and delegation (on-behalf-of) cases. 

      * Adds an /authz-store/rest/v2/actors resource for listing the indexed
        actors in long-lived (persisted) impersonated or delegated 
        authorisations.
        
      * Adds support for an optional "actor" query parameter for GET requests 
        to the /authz-store/rest/v2/authorizations resource. The "actor" query
        parameter can be used to retrieve impersonated and delegated 
        authorisations using the actor as a key.
         
      * Adds support for an optional "actor" query parameter for POST requests
        to the /authz-store/rest/v2/revocation resource. The "actor" query
        parameter can be used to revoke impersonated and delegated 
        authorisations using the actor as a key.
        

### Bug fixes

* Do not log authorisation codes, ID tokens, access tokens and refresh tokens 
  unless at log level "TRACE" (security issue server/192).

* Improves logging of failed client registration add, read, update and delete
  requests (issue server/163).


### Dependencies

* Upgrades to com.nimbusds:c2id-server-sdk:3.9

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.5

* Upgrades to com.nimbusds:oauth2-authz-store:4.0.1

* Upgrades to com.nimbusds.nimbus-jose-jwt:4.12

* Upgrades to com.nimbusds.common:1.99


### Other

* Upgrades the LDAP schema for client registrations to version 1.5:

    * Adds a new optional oauthClientData attribute for including additional
      client registration data formatted as a JSON object.
      
* Upgrades the LDAP schema for persisted authorisations to version 1.5:

    * Adds a new optional authzActor attribute for representing impersonation 
      and delegation cases.



## 3.9 (2016-02-02)

### Configuration

* /WEB-INF/oidcProvider.properties

      * Adds 'none' to the list of supported client authentication methods at
        the token endpoint (op.token.authMethods).
      

### Web API

* /clients

      * Enables explicit registration of public OAuth clients by setting the
        token_endpoint_auth_method parameter to 'none'.

* /token

      * Adds support for Proof Key for Code Exchange (PKCE) to prevent 
        authorisation code interception attacks on public OAuth clients.

* /token/revoke
      
      * Public clients are permitted to revoke access and refresh tokens that 
        have been issued to them.
        
* /authz-sessions/rest/v2

      * Adds a "client_type" field with values "confidential" and "public" to 
        the "client" object of authentication and consent prompts to indicate 
        the OAuth client type.


### Bug fixes

* Permits client registration of custom URI schemas in addition to 
  "http://localhost" for native applications (for application_type=native) 
  (issue server/187).

* Fixes unnecessary provisioning of client_secret for clients that are 
  registered for the implicit grant only (with response types "id_token" or
  "token id_token" (issue server/184).


### Dependencies

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.4

* Upgrades to com.nimbusds:oauth2-authz-store:3.5

* Upgrades to org.bouncycastle:bcprov-jdk15on:1.54


## Other

* Improves the description of client registration invalid_redirect_uri error 
  messages.

* Changes the Connect2id server policy for logging the overriding system 
  properties (if any) so that only the key names are written. This is done to
  prevent leaking of sensitive data into the server logs (issues 
  authz-store/110 and server/186).


## 3.8 (2016-01-13)

### Configuration

* /WEB-INF/oidcProvider.properties

      * Adds new op.reg.ldapServer.responseTimeout property to configure an 
        explicit timeout (in milliseconds) for responses from the LDAP 
        directory server where client registrations are persisted.
        
* /WEB-INF/authzStore.properties

      * Adds new authzStore.ldapServer.responseTimeout property to configure an 
        explicit timeout (in milliseconds) for responses from the LDAP 
        directory server where long-lived authorisations are persisted.
        
* /WEB-INF/monitor.properties

      * Adds new monitor.graphite.* properties to enable and configure 
        reporting Connect2id server metrics to a Graphite / Carbon server. 
        Includes support for prefix / API key setting, time unit conversion and 
        metric name filtering.

* /WEB-INF/web.xml
 
      * The com.nimbusds.common.servlet.MonitorLauncher servlet context
        listener is invoked last to facilitate logging of all metrics names
        for reporting via Graphite. 
      

### Web API

* /clients

      * Allows initial registration access tokens to specify the scope values
        that may be registered by the client. These are specified by giving the
        initial registration access token scope values of the form
        `client-reg:scope:[scope-value]` where `[scope-value]` denotes the
        permitted scope value that may be registered. For example, to allow 
        a client to register for the *openid* and *email* scopes, create an 
        initial registration token with the following scope:
        `client-reg:scope:openid client-reg:scope:email`.


### Bug fixes

* Ensures all paged LDAP searches that belong to a particular LDAP search
  request are performed on the same checked out LDAP connection (issues 
  authz-store/109 and server/183).


### Dependencies

* Adds dependency to io.dropwizard.metrics:metrics-graphite:3.1.2

* Upgrades to com.nimbusds:oauth2-authz-store:3.3

* Upgrades to com.nimbusds:oidc-session-store:3.3

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.1.1

* Upgrades to com.nimbusds:common:1.98.1

* Upgrades to com.thetransactioncompany:java-property-utils:1.10




## 3.7 (2015-12-28)

### Configuration

* No changes
      

### Web API

* Discovery endpoint .well-known/openid-configuration:

      * Renames OpenID Provider metadata parameter for token introspection 
        endpoint from "token_introspection_endpoint" to 
        "introspection_endpoint" to match upcoming OAuth 2.0 specification (see 
        draft-jones-oauth-discovery-00).
        
      * Renames OpenID Provider metadata parameter for token revocation 
        endpoint from "token_revocation_endpoint" to "revocation_endpoint" to 
        match upcoming OAuth 2.0 specification (see 
        draft-jones-oauth-discovery-00).
  
* Monitoring API monitor/v1/metrics:

      * Adds clientStore.ldapConnector.getTimer
      
	  * Adds clientStore.ldapConnector.searchTimer
	  
	  * Adds clientStore.ldapConnector.addTimer
	  
	  * Adds clientStore.ldapConnector.modifyTimer
	  
	  * Adds clientStore.ldapConnector.deleteTimer

      * Adds authzStore.ldapConnector.getTimer
      
	  * Adds authzStore.ldapConnector.searchTimer
	  
	  * Adds authzStore.ldapConnector.addTimer
	  
	  * Adds authzStore.ldapConnector.modifyTimer
	  
	  * Adds authzStore.ldapConnector.deleteTimer


### Bug fixes

* Ensures proper signature validation of self-issued OAuth 2.0 JWT bearer 
  assertion grants (issue server/177).

* Ensures proper shutdown of SPI implementations for handling OAuth 2.0 JWT
  bearer assertion grants issued by a third-party (issue server/178).

* Logs asynchronous authorisation store LDAP results with codes 68 (entry 
  already exists) and 32 (no such object) at info instead of error level. 
  
* Fixes AS00030 log message code (issue authz-store/107). 


### Dependencies

* Upgrades to com.nimbusds:c2id-server-sdk:3.7.1

* Upgrades to com.nimbusds:oauth2-authz-store:3.2

* Upgrades to com.nimbusds:oidc-session-store:3.2.1

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.1

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.11

* Upgrades to com.unboundid:unboundid-ldapsdk:3.1.0

* Upgrades to org.apache.commons:commons-collections4:4.1

* Upgrades to org.apache.logging.log4j:log4j-web:2.5

* Upgrades to org.apache.logging.log4j:log4j-slf4j-impl:2.5


## Other

* Adds Java Service Provider Interface (SPI) for handling SAML 2.0 bearer 
  assertion grants which are self-issued (created by the requesting client).

* Adds Java Service Provider Interface (SPI) for handling SAML 2.0 bearer 
  assertion grants which are issued by a third-party Security Token Service 
  (STS).



## 3.6 (2015-11-25)

### Configuration

* /WEB-INF/clientGrantHandler.properties

    * Reduces default access token lifetime for client credentials grant to 10 
      minutes.
      

### Web API

* Adds /token/introspect endpoint for access tokens (RFC 7662).


### Bug fixes

* Fixes inspection of expired JWT-encoded access tokens at 
  authz-store/rest/v2/inspection endpoint (issuer authz-store/106).

* Fixes OP6000 client authentication log message (issue server/168).


### Dependencies

* Upgrades to com.nimbusds:oauth2-authz-store:3.1.4

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.0-alpha12

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.7

* Upgrades to com.thetransactioncompany:cors-filter:2.5


### Other

* Writes direct authorisation messages to the Log4j "DIRECT-AUTHZ" logger.

* Writes token introspection messages to the Log4j "TOKEN-INTROSPECT" logger.

* Writes token revocation messages to the Log4j "TOKEN-REVOKE" logger.



## 3.5 (2015-11-11)

### Configuration

* /WEB-INF/oidcProvider.properties

    * Adds support for private_jwt_key client authentication at the token
      endpoint, see op.token.authMethods property.
      
    * Adds support for the following standard JWS algorithms for 
      private_jwt_key client authentication at the token endpoint, see
      op.token.authJWSAlgs property: RS256, RS384, RS512, PS256, PS384, PS512, 
      ES256, ES384, ES512

* /WEB-INF/selfIssuedJWTBearerHandler.properties

    * New configuration file for the new optional OAuth 2.0 grant handler SPI 
      for self-issued (client-issued) JWT bearer assertions (RFC 7523).
      

### Web API

* Client registration endpoint:
  
    * Supports registration of the standard "jwks" (JSON Web Key - JWK set) 
      client metadata parameter for client public RSA and EC keys, for the 
      purpose of private_key_jwt client authentication.
      
    * Initial registration access tokens can have the registration endpoint URI
      as an audience in addition to the OP issuer (issue server/167).
      
    * Returns HTTP 404 status code on a client read request with the master 
      (admin) access token if the client_id is invalid to allow privileged
      clients and administrators to distinguish between an invalid access token
      and an invalid client_id (issue server/154). Client read requests using a
      regular registration access token will still produce a HTTP 401 
      Unauthorized on an invalid client_id.


### Bug fixes

* The audience (aud) JWT claim in client_secret_jwt client authentication at
  the token endpoint must match the token endpoint URI or the OP issuer 
  identifier (issue server/157).
  
* Fixes outgoing Infinispan marshalling of subject sessions with a "claims"
  field (issue session-store/25).


### Dependencies

* Adds com.nimbusds:oauth-jwt-self-issued-grant-handler:1.0.1

* Upgrades to com.nimbusds:oauth2-authz-store:3.1.3

* Upgrades to com.nimbusds:oidc-session-store:3.2

* Upgrades to com.nimbusds:common:1.96

* Upgrades to com.nimbusds:c2id-server-sdk:3.6

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.0-alpha11

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.3.1

* Upgrades to org.bouncycastle:bcprov-jdk15on:1.53

* Upgrades to org.infinispan:infinispan-embedded:7.2.5.Final

* Upgrades to org.glassfish.jersey.containers:jersey-container-servlet:2.22.1

* Upgrades to org.apache.logging.log4j:log4j-web:2.4.1

* Upgrades to org.apache.logging.log4j:log4j-slf4j-impl:2.4.1


### Other

* Adds Service Provider Interface (SPI) for handling OAuth 2.0 JWT bearer 
  grants issued by the client (self-issued) (RFC 7523).
  
* Includes a default handler for self-issued JWT bearer grants, see
  com.nimbusds:oauth-jwt-self-issued-grant-handler:1.0.1 package.

* Adds Service Provider Interface (SPI) for handling OAuth 2.0 JWT bearer 
  grants issued by a third party (secure token service) (RFC 7523).

* Loads BouncyCastle JCA provider if advertised OP JOSE algorithms or access 
  token JOSE algorithms are not supported by the default system JCA provider(s) 
  (issue server/160).

* Provides helpful error_description message on failed client authentication;
  exact error cause is not revealed.
   


## 3.4 (2015-09-08)

### Configuration

* /WEB-INF/oidcProvider.properties
  
      * Adds support for the "code id_token token" response type which can be
        enabled in the op.authz.responseTypes configuration property. This 
        response type is intended for OpenID Connect RPs (clients) using the
        hybrid flow. See OpenID Connect Core 1.0, section 3.3 for more
        information.


### Web API

* No changes


### Dependencies

* Upgrades to com.nimbusds:c2id-server-sdk:3.3

* Upgrades to com.nimbusds:oauth2-oidc-sdk:5.0-alpha2

* Upgrades to com.nimbusds:oauth2-authz-store:3.1.2

* Upgrades to com.nimbusds:oidc-session-store:3.1.1

* Upgrades to com.nimbusds:nimbus-jose-jwt:4.3.1

* Upgrades to com.nimbusds:common:1.91

* Upgrades to org.infinispan:infinispan-embedded:7.2.4.Final

* Upgrades to org.glassfish.jersey.containers:jersey-container-servlet:2.21

* Upgrades to org.apache.logging.log4j:log4j-web:2.3

* Upgrades to org.apache.logging.log4j:log4j-slf4j-impl:2.3


### Bug fixes

* Releases LDAP connections for objectClass schema dump back to pool after use
  (issues authz-store/103 and server/144).
  
* Fixes bug which prevented inclusion of preset claims when processing UserInfo 
  requests with no authorised claim names (issue server/136).

* The Authorisation session web API handles unsupported response_mode in OpenID 
  authentication requests by returning an HTTP 220 authorisation error without
  redirection (issue server/139).

* Removes unused PresetClaimsEndpoint from /WEB-INF/web.xml (issue server/137).

  
### Other

* Includes access token hash (at_hash) and authorisation code hash (c_hash)
  claims in ID tokens only when mandated by the OpenID Connect specification 
  (issue server/151).

* Adds support for AdvancedClaimsSource SPI from 
  com.nimbusds:c2id-server-sdk:3.3 to enable passing of additional parameters 
  (client_id) to claims source connectors (issue server/150).

* Logs a UserInfo endpoint ERROR if the JWT-encoded access tokens are not 
  configured to include all required claims: sub exp cid scp clm cll uip

* Returns additional detail of client authentication "error_description" 
  messages to aid debugging. Information about the actual failure cause is not
  revealed.


## 3.3 (2015-07-28)

### Configuration

* /WEB-INF/oidcProvider.properties
  
      * Adds new op.authz.feedSubjectSessionClaimsIntoIDToken configuration 
        to control automatic feeding of claims stored in the subject (end-user) 
        session into the issued ID tokens for the subject. Applies to regular 
        as well as prompt=none OpenID Connect authentication requests.

* /WEB-INF/infinispan.xml

      * Updates the advanced-externalizer elements for the Subject Session
        Store objects.
        
* /WEB-INF/web.xml
 
      * Updates the com.nimbusds.sessionstore.rest.WebAPI servlet declaration
        for the Subject Session Store endpoint (JAX-RS).


### Web API

* Authorisation session API /authz/sessions/rest/v2:

      * The final HTTP 302 redirection response will include a 
        Subject-Session-ID header if a new subject session has been created and
        its session ID (SID) could not be communicated due to a skipped consent
        step. The Subject-Session-ID header will contain the ID of the newly
        created session, which the API caller should use to update the browser
        session cookie.

* Session Store API /session-store/rest/v2: 

      * Adds a new optional "claims" JSON object field to the session object. 
        The "claims" field may be used to store arbitrary OpenID Connect claims 
        about the subject (end-user) for inclusion into the issued ID tokens.
  
      * Adds a new /session-store/rest/v2/sessions/claims resource to allow 
        direct update (via HTTP PUT) and removal (HTTP DELETE) of the optional 
        "claims" JSON object field of a subject (end-user) session.

* Monitoring API monitor/v1/metrics:

      * Adds new sessionStore.queuedNotifications gauge to monitor the number
        of queued session start and session logout / expiration notifications
        to subscribers. This feature is not used by the Connect2id server at
        present.

### Dependencies

* Upgrades to com.nimbusds:c2id-server-sdk:3.2

* Upgrades to com.nimbusds:oidc-session-store:3.1


### Bug fixes

* Fixes a typo in the inline documentation for the 
  op.reg.ldapServer.selectionAlgorithm configuration setting. The ROUND-ROBIN
  configuration value should read ROUND_ROBIN.
  
* Fixes a typo in the inline documentation for the 
  authzStore.ldapServer.selectionAlgorithm configuration setting. The 
  ROUND-ROBIN configuration value should read ROUND_ROBIN.
  
  


## 3.2.1 (2015-07-08)

### Configuration

* /WEB-INF/oidcProvider.properties
  
      * Extends the op.authz.alwaysPromptForConsent setting to apply to 
        OpenID Connect prompt=none requests. A consent_required error will no
        longer be produced when this setting is enabled and a client requests
        an ID token refresh / user session check with prompt=none. 

### Web API

* No changes

### Dependencies

* Upgrades to com.nimbusds:oauth2-oidc-sdk:4.14.1

* Upgrades to com.nimbusds:oidc-session-store:2.2.1

### Bug fixes

* Fixes the URL encoding of login_required and consent_required OpenID Connect
  authentication request errors in the authorisation code flow (issue 
  server/130).
  
* Fixes deserialisation of subject identifiers with colons in cluster mode 
  (issue session-store/15).



## 3.2 (2015-07-02)

### Configuration

* No changes
      
### Web API
      
* Requires an all=true query parameter to delete all end-user sessions at the
  /session-store/rest/v2/sessions resource. Intended to prevent accidental
  deletion of all sessions.
  
### Dependencies

* Upgrades to com.nimbusds:oauth2-authz-store:3.0.5

* Upgrades to com.nimbusds:oidc-session-store:2.1

* Upgrades to com.nimbusds:common:1.90

* Upgrades to com.unboundid:unboundid-ldapsdk:3.0.0

* Upgrades to org.glassfish.jersey.containers:jersey-container-servlet:2.19

* Upgrades to io.dropwizard.metrics:metrics-core:3.1.2

* Upgrades to io.dropwizard.metrics:metrics-healthchecks:3.1.2

* Upgrades to io.dropwizard.metrics:metrics-servlets:3.1.2

### Other

* Adds JGroups configuration template for S3-ping node discovery 
  resources/default-jgroups-s3ping.xml

* Logs LDAP schemas of required object classes on server startup to ease
  debugging.


## 3.1.1 (2015-06-30)

### Configuration

* No changes
      
### Web API
      
* No changes 
  
### Dependencies

* Upgrades to com.nimbusds:c2id-server-ldap-schemas:1.4.1
* Upgrades to org.infinispan:infinispan-embedded:7.2.3.Final

### Bug fixes

* Fixes LDAP schema for client registrations to allow multi-valued 
  post_logout_redirect_uri attribute values (issue server/128).


## 3.1 (2015-06-22)

### Configuration

* /WEB-INF/oidcProvider.properties

    * The advertised Authentication Context Class References (ACRs) specified
      by op.authz.advertisedACRs (if any) should be ordered from weakest to 
      strongest level. This ordering will affect processing of OpenID Connect
      authentication requests with a given ACR.
      
### Web API
      
* The Authorisation session endpoint at /authz-sessions/rest/v2 assumes will 
  not prompt for re-authentication if the requested Authentication Context 
  Class Reference (ACR, or authentication strength) is exceeded by the ACR for
  the current subject session. The relative strength of the current and the
  requested ACR is determined by the order of the supported ACR values
  advertised in the op.authz.advertisedACRs configuration setting. Previously
  the Connect2id server would limit ACR re-authentication checks to exact 
  matches only and would leave relative checks to be taken care of by the 
  login page (business logic).
      

### Dependencies

* Upgrades to com.nimbusds:oauth2-authz-store:3.0.4


### Bug fixes

* Fixes handling of OpenID Connect authentication requests with prompt=login
  (issue server/126).
  
* Fixes refresh token return on direct authorisation submission when the 
  authorisation matches an existing authorisation record (issue 
  authz-store/98).


## 3.0 (2015-03-26)

### Configuration

* /WEB-INF/jwkSet.json

    * Adds support for specifying an optional octet-sequence JSON Web Key (JWK)
      in the key set for applying additional symmetric encryption of
      self-contained (JWT-encoded) access tokens.

* /WEB-INF/oidcProvider.properties

    * Renames op.idToken.lifetime to op.idToken.defaultLifetime and changes the
      value to signify seconds instead of minutes. The value can be overridden
      by individual authorisations via the integration APIs.

    * Changes time unit of op.authz.sessionLifetime to seconds.

    * Adds an op.authz.includeClientInfoInAuthPrompt configuration property to
      include the registered OpenID Connect client information in the
      authentication prompt. The client details may then be used as an
      additional input to the determine the appropriate end-user authentication
      or session settings.

    * Adds an op.authz.includeOtherConsentedScopeAndClaimsInPrompt
      configuration property to include non-requested scope values and claim
      names for which previous consent exists in the consent prompt.

    * Renames the op.authz.promptNone.alwaysRequireConsent configuration
      property to op.authz.alwaysPromptForConsent and changes its semantics to
      apply to all OpenID authentication requests. If true the Connect2id
      server will always prompt for consent, even if the end-user is logged in
      and previous consent is has been recorded; OpenID authentication with
      prompt=none will produce a consent_required error. If false the
      Connect2id server will immediately return a successful redirection URI
      response provided the end-user is logged in and previous consent exists
      (as with prompt=none).

    * Removes the op.authz.promptNone.alwaysRequireLogin configuration
      property. OpenID authentication requests with prompt=none will always
      succeed if the end-user is logged in, previous consent exists and
      op.authz.alwaysPromptForConsent is set to false.

    * Renames the op.authz.promptNone.requireIDTokenHint configuration property
      to op.authz.requireIDTokenHintWithPromptNone.

* /WEB-INF/authzStore.properties

    * Renames the authzStore.accessToken.lifetime configuration property to
      authzStore.accessToken.defaultLifetime. The value can be overridden by
      individual authorisations via the integration APIs.

    * Removes the authzStore.accessToken.defaultEncoding configuration
      property. The encoding of all access tokens will default to self-
      contained (JWT-encoded) unless overridden by individual authorisations
      via the integration APIs.

    * Adds a authzStore.accessToken.jweAlgorithm configuration property to
      specify the optional JSON Web Encryption (JWE) algorithm to apply to
      self-contained (JWT-encoded) access tokens. The only support algorithm is
      'dir' (direct encryption with a shared symmetric key.

    * Adds a authzStore.accessToken.jweMethod configuration property to specify
      the optional JSON Web Encryption (JWE) method to apply to self-contained
      (JWT-encoded) access tokens. Supported methods are A128GCM (recommended),
      A192GCM, A256GCM, A128CBC_HS256, A192CBC_HS384 and A256CBC_HS512.

    * All claims specified in the authzStore.accessToken.selfContainedClaims
      configuration property are treated as optional for inclusion in
      self-contained (JWT-encoded) access tokens.

    * The authzStore.ldapDirectory.attributes.rts configuration property adds
      support for defining an LDAP attribute alias.

    * Adds a authzStore.ldapDirectory.attributes.rtl configuration property to
      specify the name of the LDAP attribute holding the refresh token
      lifetime, in seconds.

    * Adds a authzStore.ldapDirectory.attributes.rti configuration property to
      specify the name of the LDAP attribute holding the refresh token issue
      date.

    * Adds a authzStore.ldapDirectory.attributes.atc configuration property to
      specify the name of the LDAP attribute holding the access token encrypt
      setting.

* Adds a new /WEB-INF/claimsCompression.properties configuration file to
  specify an optional claim names compression map for self-contained (JWT)
  access tokens.

* Adds a new /WEB-INF/monitor.properties configuration file to specify an OAuth
  2.0 bearer access token for the new metrics and health-check endpoint.

* /WEB-INF/clientGrantHandler.properties

    * Adds a op.grantHandler.clientCredentials.simpleHandler.accessToken.encrypt
      configuration property to enable encryption of self-contained (JWT-
      encoded) access tokens.

* /WEB-INF/infinispan.xml

    * Updates the authorisation store externalisers.
  

### Web API

* Adds a /clients alias to the /client-reg client registration endpoint.

* Introduces a new version of the Authorisation session endpoint at
  /authz-sessions/rest/v2. 
  
  Extends the authentication prompt object as follows:
  
    * The registered details for the requesting client will be included if the
      op.authz.includeClientInfoInAuthPrompt configuration property is set.
  
  Extends and modifies the consent object as follows:

    * Adds a new optional id_token parameter to enable the default ID token
      lifetime to be overridden and to set an impersonated subject (the
      original subject will be set in a custom authz_sub ID token claim).

    * Moves the optional issue_refresh_token parameter into a new containing
      refresh_token parameter which also enables an optional lifetime for the
      refresh token to be set (zero lifetime implies permanent).

    * Adds a new optional access_token encrypt parameter to additionally
      encrypt self-contained (JWT-encoded) access tokens with a shared
      symmetric AES key.

    * Adds an optional data parameter for additional information to be stored
      in the authorisation record and self-contained (JWT-encoded) access tokens.

  The original version of the Authorisation session endpoint is still available
  at /authz-sessions/rest/v1.

* The Authorisation session endpoint will bypass consent and immediately return
  a successful redirection URI response if the end-user is logged in, previous
  consent exists and the op.authz.alwaysPromptForConsent configuration
  property is set to false. This behaviour applies to versions 1 and 2 of the
  Authorisation session endpoint.

* Introduces a new version of the Direct authorisation endpoint at
  /direct-authz/rest/v2. Extends and modifies the direct authorisation request
  as follows:

    * Adds a new optional id_token parameter to enable the default ID token
      lifetime to be overridden and to set an impersonated subject (the
      original subject will be set in a custom authz_sub ID token claim).

    * Moves the optional issue_refresh_token parameter into a new containing
      refresh_token parameter which also enables an optional lifetime for the
      refresh token to be set (zero lifetime implies permanent).

    * Adds a new optional access_token encrypt parameter to additionally
      encrypt self-contained (JWT-encoded) access tokens with a shared
      symmetric AES key.

    * Adds an optional data parameter for additional information to be stored
      in the authorisation record and self-contained (JWT-encoded) access
      tokens.

  The original version of the Direct authorisation session endpoint is still
  available at /direct-authz/rest/v1.

* Introduces a new version of the Session store endpoint at
  /session-store/rest/v2. To prevent leakage of session identifiers (SID) into
  client and server logs for GET and other HTTP requests these are no longer
  specified as a path component; a SID request header parameter is used
  instead. Version 1 of the Session store endpoint is not longer supported.

* Introduces a new version of the Authorisation store endpoint at
  /authz-store/rest/v2 to reflect underlying architectural changes. Summary of
  the changes:

    * Adds a new /authz-store/rest/v2/inspection resource to introspect
      individual authorisation codes, access tokens and refresh tokens via
      HTTP POST.

    * The previous access-tokens resource for returning all current access
      tokens and for introspecting individual access tokens is removed. Use of
      the new inspection resource is suggested to introspect individual access
      tokens instead.

    * The authorizations resource no longer supports the access_token or
      refresh_token query parameters to prevent leakage of these token
      credentials into client and server logs for GET HTTP requests. Use of the
      new inspection resource is suggested instead.

    * Adds a new /authz-store/rest/v2/revocation resource to revoke one or more
      authorisations by access token, refresh token, by subject and / or by
      client identifier via HTTP POST.

    * Revocation of authorisations by HTTP DELETE on the authorizations
      resource is no longer supported. Use of the new revocation resource is
      suggested instead.

    * Revocation of individual access tokens without affecting the underlying
      authorisation is no longer supported.

    * Revocation of individual refresh tokens without affecting the underlying
      authorisation is not longer supported.

  Version 1 of the Authorisation store endpoint is no longer supported.

* Adds a new metrics and health-check endpoint at /monitor/v1 with over 40 data
  points for usage, request performance and throughput, Infinispan cache and
  LDAP connection pool. The endpoint is protected by means of an OAuth 2.0
  Bearer token and based on the Dropwizard Metrics library.

* Adds 15K character limit on client registration endpoint HTTP POST and PUT
  methods to guard against DoS attacks.

* Adds 2K character limit on the token endpoint HTTP POST method to guard
  against DoS attacks.


### Dependencies

* Adds io.dropwizard.metrics:metrics-core:3.1.1

* Adds io.dropwizard.metrics:metrics-healthchecks:3.1.1

* Adds io.dropwizard.metrics:metrics-servlets:3.1.1

* Upgrades to com.nimbusds:oauth2-oidc-sdk:4.12.1

* Upgrades to com.nimbusds:c2id-server-sdk:3.0

* Upgrades to com.nimbusds:oidc-claims-source-ldap:1.4.1

* Upgrades to com.nimbusds:oauth-password-grant-web-api:1.3

* Upgrades to com.nimbusds:oauth-client-grant-handler:1.3

* Upgrades to com.nimbusds:oauth2-authz-store:3.0.3

* Upgrades to com.nimbusds:oidc-session-store:2.0.3

* Upgrades to com.nimbusds:nimbus-jose-jwt:3.9.2

* Upgrades to com.nimbusds:common:1.89.4

* Upgrades to net.minidev:json-smart:1.3.1

* Upgrades to com.thetransactioncompany:cors-filter:2.3

* Upgrades to com.unboundid:unboundid-ldapsdk:2.3.8

* Upgrades to org.infinispan:infinispan-embedded:7.1.1.Final

* Upgrades to rg.glassfish.jersey.containers:jersey-container-servlet:2.17


### Bug fixes

* Returns a meaningful error response message on missing openid scope value in
  submitted consent to the Authorisation session endpoint (issue server/83).

* Always returns consent prompt on a OpenID Connect authentication request with
  prompt=consent or prompt=select_account (issue server/100).


### Other

* Upgrades the LDAP schema for persisted authorisations to version 1.4:

    * Adds an authzRefreshTokenSecret alias to authzRefreshTokenSalt, promotes
      the former to main attribute name.

    * Adds authzRefreshTokenLifetime attribute.

    * Adds authzRefreshTokenIssueDate attribute.



## 2.5 (2015-03-02)

### Configuration

* Core provider configuration: Adds op.reg.refreshAccessTokenOnUpdate
  configuration property to enable / disable refreshing of the registration
  access token with each client registration update. The previous Connect2id
  server policy was to always refresh the registration access token on client
  updates.

* Core provider configuration: Adds op.reg.alwaysRefreshClientSecretOnUpdate
  configuration property to enable / disable refreshing of the client secret
  with each client registration update. If false the client secret will be
  refreshed only if it has expired. The previous Connect2id server policy was
  to always refresh the secret on client updates.

* Core provider configuration: Removes the op.reg.clientIDByteLength
  configuration property in favour of using an internal lookup table to
  determine the minimal appropriate client secret length based on the chosen
  client authentication method, ID token JWS algorithm and UserInfo JWS
  algorithm at client registration time.

### Web API

* Client registration API: Adds support for a custom preferred_client_secret
  registration parameter. Enables registration of clients with a preset secret.
  Requires the master access token or an initial registration token with the
  client-reg:set-secret scope value. The secret length must satisfy the minimal
  requirement for the chosen client authentication method, ID token JWS
  algorithm and UserInfo JWS algorithm (256, 384 or 512 bits).


### Dependencies

* No changes


### Bug fixes

* Removes preferred_client_id from client information responses (issue
  server/124).

* Fixes empty response_type on client registration with 'code token id_token'
  (issue server/111).

* Ensures minimum client secret length as per JSON Web Signature (JWS) / HMAC
  specification (server/104).



## 2.4.4 (2015-02-24)

### Configuration

* No changes


### Web API

* No changes


### Dependencies

* No changes


### Bug fixes

* Issue server/119: Fixes a missing "exp" claim in self-contained access tokens
  issued at the direct authorisation endpoint when no access token lifetime is
  specified.



## 2.4.3 (2014-12-11)

### Configuration

* No changes


### Web API

* No changes


### Dependencies 

* No changes


### Bug fixes 

* Fixes a bug that affected client_id parsing during client registration
  retrieval via the web API.


## 2.4.2 (2014-12-07)

### Configuration

* No changes


### Web API

* No changes


### Dependencies 

* Upgrades to com.nimbusds:oidc-session-store:1.3.1


### Bug fixes 

* Removes a stray System.out.println from the Infinispan subject session
  serialiser.
  


## 2.4.1 (2014-12-04)

### Configuration

* No changes


### Web API

* No changes


### Dependencies 

* Upgrades to com.nimbusds:oauth2-authz-store.2.7.1


### Bug fixes 

* Fixes an authorisation store bug that affected Infinispan record 
  serialisation.


## 2.4 (2014-11-24)

### Configuration

* Support configuration override from system properties (using the optional -D 
  command line arguments).


### Web API

* No changes


### Dependencies

* Upgrades to com.nimbusds:nimbus-jose-jwt:3.4

* Upgrades to com.nimbusds:oauth2-oidc-sdk:4.8

* Upgrades to com.nimbusds:common:1.83.1

* Upgrades to com.nimbusds:oidc-session-store:1.3

* Upgrades to com.nimbusds:oauth2-authz-store:2.7

* Upgrades to com.nimbusds:c2id-server-sdk:2.0.7

* Upgrades to com.nimbusds:oidc-claims-source-ldap:1.3

* Upgrades to com.nimbusds:oauth-password-grant-web-api:1.2

* Upgrades to com.nimbusds:oauth-client-grant-handler:1.2

* Upgrades to org.infinispan:infinispan-core:7.0.2.Final


### Bug fixes

* Fixes Infinispan serialisation of the client secret expiration in OpenID
  Connect client registrations when the value is set to zero (no expiration).
  
  
  
## 2.3.2 (2014-11-17)

### Configuration

* No changes 


### Web API

* Authorisation store: Removes deprecated `sid` field for the optional end-user
  session identifier field. 


### Dependencies

* Upgrades to com.nimbusds:oauth2-authz-store:2.6


### Bug fixes

* Returns HTTP 403 Forbidden / insufficient_scope at UserInfo endpoint when 
  offline_access is not granted. 
  
* Ceases use of LDAP permissive modify request control (OID 
  1.2.840.113556.1.4.1413). 
